/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deploy.deployable.test.xmlhelper;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

import java.io.File;

import org.ow2.util.ee.deploy.impl.deployable.factory.utils.XmlHelper;
import org.testng.annotations.Test;
import org.xml.sax.SAXException;

/**
 * Test class for the xml helper.
 * @author mleduque
 */
public class XmlCheckerTest {

    private static final String DEPPLAN_NAMESPACE =
        "http://jonas.ow2.org/ns/deployment-plan/1.0";

    /**
     * The xml helper.
     */
    private XmlHelper helper = new XmlHelper();

    /**
     * Constructor (needed because of the exception).
     * @throws Exception if the XmlHelper construction fails.
     */
    public XmlCheckerTest() throws Exception {

    }

    /**
     * Checks the isWellFormedXml on a well-formed XML.
     * @throws Exception
     */
    @Test
    public void parseGood() throws Exception {
        assertTrue(helper.isWellFormedXml(
                new File("target/test-classes/deployment-plan.xml")));
    }

    /**
     * Checks the isWellFormedXml on a not well-formed XML.
     * @throws Exception
     */
    @Test
    public void parseBadFile() throws Exception {
        assertFalse(helper.isWellFormedXml(
                new File("target/test-classes/bad.xml")));
    }

    /**
     * Checks the isWellFormedXml on a file that is not XML.
     * @throws Exception
     */
    @Test
    public void parseNotXml() throws Exception {
        assertFalse(helper.isWellFormedXml(
                new File("target/test-classes/notxml.xml")));
    }

    /**
     * Checks the isWellFormedXml on a file that has an XML header but
     * is not XML.
     * @throws Exception
     */
    @Test
    public void parseFalseXml() throws Exception {
        assertFalse(helper.isWellFormedXml(
                new File("target/test-classes/falsexml.xml")));
    }

    /**
     * Checks the isLikelyXml on a well-formed XML.
     * @throws Exception
     */
    @Test
    public void parseLikelyGood() throws Exception {
        assertTrue(helper.isLikelyXml(
                new File("target/test-classes/deployment-plan.xml")));
    }

    /**
     * Checks the isLikelyXml on a not well-formed XML.
     * @throws Exception
     */
    @Test
    public void parseLikelyBadFile() throws Exception {
        assertTrue(helper.isLikelyXml(
                new File("target/test-classes/bad.xml")));
    }

    /**
     * Checks the isLikelyXml on a non XML file.
     * @throws Exception
     */
    @Test
    public void parseLikelyNotXml() throws Exception {
        assertFalse(helper.isLikelyXml(
                new File("target/test-classes/notxml.xml")));
    }

    /**
     * Checks the isLikelyXml on a non XML file with XML header.
     * @throws Exception
     */
    @Test
    public void parseLikelyFalseXml() throws Exception {
        assertFalse(helper.isLikelyXml(
                new File("target/test-classes/falsexml.xml")));
    }

    /**
     * Checks the getRootElementNameSpace on a well-formed XML.
     * @throws Exception
     */
    @Test
    public void getRootNamespaceGood() throws Exception {
        assertEquals(helper.getRootElementNameSpace(
                new File("target/test-classes/deployment-plan.xml")),
                DEPPLAN_NAMESPACE);
    }

    /**
     * Checks the getRootElementNameSpace on a not well-formed XML.
     * @throws Exception
     */
    @Test
    public void getRootNamespaceBad() throws Exception {
        assertEquals(helper.getRootElementNameSpace(
                new File("target/test-classes/bad.xml")),
                "");
    }

    /**
     * Checks the getRootElementNameSpace on a non XML file.
     * @throws Exception
     */
    @Test(expectedExceptions = { SAXException.class })
    public void getRootNamespaceNotXml() throws Exception {
        helper.getRootElementNameSpace(
                new File("target/test-classes/notxml.xml"));
    }

    /**
     * Checks the getRootElementNameSpace on a non XML file
     * with an XML header.
     * @throws Exception
     */
    @Test(expectedExceptions = { SAXException.class })
    public void getRootNamespaceFalseXml() throws Exception {
        helper.getRootElementNameSpace(
                new File("target/test-classes/falsexml.xml"));
    }

}
