/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deployer;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptEngineManager;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.util.ee.deploy.api.deployer.IDeployerManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.deployer.api.IDeploymentPlanDeployer;
import org.ow2.util.plan.deployer.api.IRepositoryDeployer;
import org.ow2.util.plan.deployer.api.condition.IConditionEvaluator;
import org.ow2.util.plan.deployer.impl.DeploymentPlanDeployer;
import org.ow2.util.plan.deployer.impl.condition.ScriptConditionEvaluator;
import org.ow2.util.plan.deployer.script.OSGiAwareScriptEngineFactory;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactoryManager;
import org.ow2.util.plan.monitor.api.IResourceMonitor;
import org.ow2.util.plan.reader.plan.IPlanReader;
import org.ow2.util.plan.reader.repository.IRepositoryDataReader;
import org.ow2.util.plan.repository.api.IRepositoryManager;



/**
 * Activator for the deployer bundle.
 * @author Mickaël LEDUQUE
 *
 */
public class PlanDeployerActivator implements BundleActivator, ServiceListener {
    /**
     * The logger.
     */
    private Log logger = LogFactory.getLog(PlanDeployerActivator.class);

    /**
     * The bundle context
     */
    private BundleContext bundleContext = null;

    /**
     * The services the activator listens to
     */
    private Class<?>[] services = {IPlanReader.class,
                                IResourceFetcherFactoryManager.class,
                                IResourceMonitor.class,
                                IRepositoryDataReader.class,
                                IRepositoryManager.class,
                                IDeployerManager.class
                                };

    /**
     * The deployment plan deployer.
     */
    private DeploymentPlanDeployer deploymentPlanDeployer = null;

    /**
     * The repository deployer.
     */
    private OsgiRepositoryDeployerImpl repositoryDeployer = null;

    /**
     * Service registration for the deployment plan deployer.
     */
    private ServiceRegistration deploymentPlanDeployerRegistration = null;

    /**
     * Service registration for the drepository deployer.
     */
    private ServiceRegistration repositoryDeployerRegistration = null;

    /**
     * The deployer manager.
     */
    private IDeployerManager deployerManager = null;

    /**
     * {@inheritDoc}
     */
    public void start(BundleContext context) throws Exception {
        this.bundleContext = context;
        deploymentPlanDeployer = new DeploymentPlanDeployer();
        deploymentPlanDeployer.setConditionEvaluator(createConditionEvaluator());
        repositoryDeployer = new OsgiRepositoryDeployerImpl();
        deploymentPlanDeployerRegistration = context.registerService(
                IDeploymentPlanDeployer.class.getName(),
                deploymentPlanDeployer, null);
        repositoryDeployerRegistration = context.registerService(
                IRepositoryDeployer.class.getName(),
                repositoryDeployer, null);

        /* Deployment plan deployer initialization */
        /* plan reader */
        ServiceReference planReaderRef = context.getServiceReference(IPlanReader.class.getName());
        if (planReaderRef != null) {
            IPlanReader reader = (IPlanReader)context.getService(planReaderRef);
            deploymentPlanDeployer.setPlanReader(reader);
            logger.debug("The deployment plan XML reader was set to {0}", reader);
        }
        /* fetcher factories manager */
        ServiceReference factoriesManagerRef = context.getServiceReference(IResourceFetcherFactoryManager.class.getName());
        if (factoriesManagerRef != null) {
            IResourceFetcherFactoryManager manager =
                (IResourceFetcherFactoryManager)context.getService(factoriesManagerRef);
            deploymentPlanDeployer.setFetcherFactoryManager(manager);
            logger.debug("The resource fetcher factory manager was set to {0}", manager);
        }

        /* resource monitor */
        ServiceReference resourceMonitorRef = context.getServiceReference(IResourceMonitor.class.getName());
        if (resourceMonitorRef != null) {
            IResourceMonitor monitor =
                (IResourceMonitor)context.getService(resourceMonitorRef);
            deploymentPlanDeployer.setResourceMonitor(monitor);
            logger.debug("The resource monitor was set to {0}", monitor);
        }

        /* Repositories deployer initialization */
        /* Reader */
        ServiceReference repositoryReaderRef = context.getServiceReference(IRepositoryDataReader.class.getName());
        if (repositoryReaderRef != null) {
            IRepositoryDataReader reader = (IRepositoryDataReader)context.getService(repositoryReaderRef);
            repositoryDeployer.setReader(reader);
            logger.debug("The repositories file XML reader was set to {0}", reader);
        }
        /* RepositoryManager */
        ServiceReference repositoryManagerRef = context.getServiceReference(IRepositoryManager.class.getName());
        if (repositoryManagerRef != null) {
            IRepositoryManager manager = (IRepositoryManager)context.getService(repositoryManagerRef);
            repositoryDeployer.setRepositoryManager(manager);
            logger.debug("The repositories manager was set to {0}", manager);
        }

        /* Register to the DeployerManager */
        ServiceReference deployerManagerRef = context.getServiceReference(IDeployerManager.class.getName());
        if (deployerManagerRef != null) {
            IDeployerManager deployerManager = (IDeployerManager)context.getService(deployerManagerRef);
            deployerManager.register(this.deploymentPlanDeployer);
            deployerManager.register(this.repositoryDeployer);
            deploymentPlanDeployer.setDeployerManager(deployerManager);
            this.deployerManager = deployerManager;
            logger.debug("The deployer manager was set to {0}", deployerManager);
        }

        // Register as listener.
        context.addServiceListener(this, buildListenerString());
    }

    private IConditionEvaluator createConditionEvaluator() {
        ScriptConditionEvaluator scriptConditionEvaluator = new ScriptConditionEvaluator();
        scriptConditionEvaluator.setScriptEngineFactory(createScriptEngineFactory());
        return scriptConditionEvaluator;
    }

    private ScriptEngineFactory createScriptEngineFactory() {
        OSGiAwareScriptEngineFactory scriptEngineFactory = new OSGiAwareScriptEngineFactory(bundleContext);
        scriptEngineFactory.setScriptEngineFactory(getPlatformJavaScriptEngineFactory());
        return scriptEngineFactory;
    }

    private ScriptEngineFactory getPlatformJavaScriptEngineFactory() {
        ScriptEngineManager manager = new ScriptEngineManager(ClassLoader.getSystemClassLoader());
        ScriptEngine engine = manager.getEngineByName("JavaScript");
        return engine.getFactory();
    }

    /**
     * {@inheritDoc}
     */
    public void stop(BundleContext context) throws Exception {
        if (deployerManager != null) {
            deployerManager.unregister(this.deploymentPlanDeployer);
            deployerManager.unregister(this.repositoryDeployer);
        }
        if (deploymentPlanDeployerRegistration != null) {
            deploymentPlanDeployerRegistration.unregister();
        }
        if (repositoryDeployerRegistration != null) {
            repositoryDeployerRegistration.unregister();
        }
    }

    /**
     * {@inheritDoc}
     * For the each registered service, we check if the service is already
     * affected, and only when it is not, we affect it.
     */
    public void serviceChanged(ServiceEvent event) {
        ServiceReference serviceRef = event.getServiceReference();
        Object serviceObject = this.bundleContext.getService(serviceRef);


        if (serviceObject instanceof IPlanReader) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.deploymentPlanDeployer.getPlanReader() == null) {
                    IPlanReader planReader = (IPlanReader)serviceObject;
                    this.deploymentPlanDeployer.setPlanReader(planReader);
                    logger.debug("The deployment plan XML reader was set to {0}", planReader);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.deploymentPlanDeployer.setPlanReader(null);
                logger.debug("The deployment plan XML reader was unset");
                break;
            default :
                break;
            }
        }

        if (serviceObject instanceof IResourceFetcherFactoryManager) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.deploymentPlanDeployer.getFetcherFactoryManager() == null) {
                    IResourceFetcherFactoryManager resourceFetcherFactoryManager =
                                (IResourceFetcherFactoryManager)serviceObject;
                    this.deploymentPlanDeployer.setFetcherFactoryManager(resourceFetcherFactoryManager);
                    logger.debug("The resource fetcher factory manager was set to {0}", resourceFetcherFactoryManager);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.deploymentPlanDeployer.setFetcherFactoryManager(null);
                logger.debug("The resource fetcher factory manager was unset");
                break;
            default :
                break;
            }
        }

        if (serviceObject instanceof IResourceMonitor) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.deploymentPlanDeployer.getResourceMonitor() == null) {
                    IResourceMonitor resourceMonitor = (IResourceMonitor)serviceObject;
                    this.deploymentPlanDeployer.setResourceMonitor(resourceMonitor);
                    logger.debug("The resource monitor was set to {0}", resourceMonitor);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.deploymentPlanDeployer.setResourceMonitor(null);
                logger.debug("The resource monitor was unset");
                break;
            default :
                break;
            }
        }

        if (serviceObject instanceof IRepositoryDataReader) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.repositoryDeployer.getReader() == null) {
                    IRepositoryDataReader repositoryReader = (IRepositoryDataReader)serviceObject;
                    this.repositoryDeployer.setReader(repositoryReader);
                    logger.debug("The repository XML reader was set to {0}", repositoryReader);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.repositoryDeployer.setReader(null);
                logger.debug("The repository XML reader was unset");
                break;
            default :
                break;
            }
        }

        if (serviceObject instanceof IRepositoryManager) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.repositoryDeployer.getRepositoryManager() == null) {
                    IRepositoryManager repositoryManager = (IRepositoryManager)serviceObject;
                    this.repositoryDeployer.setRepositoryManager(repositoryManager);
                    logger.debug("The repository manager was set to {0}", repositoryManager);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.repositoryDeployer.setRepositoryManager(null);
                logger.debug("The repository manager was unset");
                break;
            default :
                break;
            }
        }

        if (serviceObject instanceof IDeployerManager) {
            switch (event.getType()) {
            case ServiceEvent.REGISTERED :
                if (this.deployerManager == null) {
                    this.deployerManager = (IDeployerManager)serviceObject;
                    logger.debug("The deployer manager was set to {0}", this.deployerManager);
                }
                break;
            case ServiceEvent.UNREGISTERING :
                this.deployerManager = null;
                logger.debug("The deployer manager was unset");
                break;
            default :
                break;
            }
        }
    }

    /**
     * Builds the string that will be used for service listening.
     * @return
     */
    private String buildListenerString() {
        StringBuilder sb = new StringBuilder("(|");
        for (Class<?> clazz : services) {
            sb.append(buildListenerStringForService(clazz.getName()));
        }
        sb.append(")");

        return sb.toString();
    }

    /**
     * Builds the part of the listener registration string that relates
     * to one service.
     * @param service the service (full) class name.
     * @return the string for this service.
     */
    private String buildListenerStringForService(String service) {
        StringBuilder sb = new StringBuilder("(");
        sb.append(Constants.OBJECTCLASS);
        sb.append("=");
        sb.append(service);
        sb.append(")");

        return sb.toString();
    }
}
