/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.fetcher.impl.url;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.fetcher.api.IResourceFetcherFactory;
import org.ow2.util.plan.repository.api.IRepositoryManager;


/**
 * Activator for the fetcher/url bundle.
 * It registers a UrlResourceFetcherFactory, registers it as listener for
 * url protocol registration/unregistration.
 * @author Mickaël LEDUQUE
 *
 */
public class UrlFetcherActivator implements BundleActivator, ServiceListener {

    /**
     * The logger.
     */
    private Log logger = LogFactory.getLog(UrlFetcherActivator.class);

    /**
     * The fetcher factory manager.
     */
    UrlResourceFetcherFactory fetcherFactory = null;

    /**
     * The ServiceRegistration object for the registered XmlHelper object.
     */
    private ServiceRegistration urlFetcherFactoryServiceRegistration = null;

    /**
     * The listener filter for this bundle.
     */
    private final static String LISTENER_FILTER_STRING = 
        "(" + Constants.OBJECTCLASS + "=" + IRepositoryManager.class.getName() + ")";

    /**
     * The bundle context.
     */
    private BundleContext bundleContext = null;

    /**
     * {@inheritDoc}
     * 
     * Specific : sets up and registers a Url resource fetcher.
     */
    public void start(BundleContext context) throws Exception {
        this.bundleContext = context;
        fetcherFactory = new UrlResourceFetcherFactory();
        /* registers self as service events listener */
        context.addServiceListener(this,LISTENER_FILTER_STRING);

        /* Set up the repository manager */
        ServiceReference repositoryManagerRef = context.getServiceReference(
                IRepositoryManager.class.getName());
        if (repositoryManagerRef != null) {
            IRepositoryManager repositoryManager = (IRepositoryManager)context.getService(repositoryManagerRef);
            fetcherFactory.setRepositoryManager(repositoryManager);
            logger.debug("the repository manager is set to " + repositoryManager);
        }
        /* registers the factory as both fetcher factory and protocol manager */
        this.urlFetcherFactoryServiceRegistration = context.registerService(
                        IResourceFetcherFactory.class.getName(),
                        fetcherFactory, null);

    }

    /**
     * {@inheritDoc}
     */
    public void stop(BundleContext context) throws Exception {
        urlFetcherFactoryServiceRegistration.unregister();
        logger.debug("URL fetcher factory unregistered.");
    }

    /**
     * {@inheritDoc}
     * 
     * Specific : listens to repository manager registration
     * and unregistration events.
     */
    public void serviceChanged(ServiceEvent event) {
        /* Will process fetcher factories registration and unregistration events
         * to maintain factory list */
        ServiceReference serviceRef = event.getServiceReference();
        Object serviceObject = bundleContext.getService(serviceRef);
        if (serviceObject instanceof IRepositoryManager) {
            IRepositoryManager repositoryManager = (IRepositoryManager) serviceObject;
            switch (event.getType()) {
                case ServiceEvent.REGISTERED :
                    if (fetcherFactory.getRepositoryManager() == null) {
                        fetcherFactory.setRepositoryManager(repositoryManager);
                        logger.debug("The repository manager is set to '{0}'.", repositoryManager);
                    }
                    break;
                case ServiceEvent.UNREGISTERING :
                    fetcherFactory.setRepositoryManager(null);
                    logger.debug("The repository manager is unset.");
                    break;
                default : break;
            }
        }
    }

}
