/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.reader.plan;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.validation.SchemaFactory;

import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlan;
import org.ow2.util.plan.reader.AbsReader;
import org.ow2.util.plan.reader.ReaderException;
import org.xml.sax.InputSource;

/**
 * Implementation for the IPlanReader interface.
 * @author Mickaël LEDUQUE
 */
public class PlanReaderImpl extends AbsReader implements IPlanReader {

    /**
     * location of the deployment plan schema resources.
     */
    private static final String DEPLOYMENT_PLAN_XSD = "deployment-plan-1.0.xsd";
    private static final String MAVEN2_DEPLOYMENT_PLAN_XSD = "maven2-deployment-plan-1.0.xsd";
    private static final String URL_DEPLOYMENT_PLAN_XSD = "url-deployment-plan-1.0.xsd";
    private static final String OBR_DEPLOYMENT_PLAN_XSD = "obr-deployment-plan-1.0.xsd";

    /**
     * XML binding classes used by PlanReaders.
     */
    private static final Class[] PLAN_CLASSES = {
            org.ow2.util.plan.bindings.deploymentplan.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.maven2.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.obr.ObjectFactory.class,
            org.ow2.util.plan.bindings.deploymentplan.url.ObjectFactory.class };

    /**
     * {@inheritDoc}
     */
    public PlanReaderImpl() throws Exception {
        super();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void initUnmarshallClasses() {
        unmarshallClasses = PLAN_CLASSES;
    }

    /**
     * {@inheritDoc}
     */
    public synchronized DeploymentPlan readPlan(final File file) throws IOException,
            ReaderException {

        JAXBElement jaxbElement = null;
        try {
            jaxbElement = getRootElement(file);
        } catch (JAXBException e) {
            throw new ReaderException("Unable to parse the file '" + file + "'", e);
        }

        Object elementValue = jaxbElement.getValue();
        if (elementValue instanceof DeploymentPlan) {
            return (DeploymentPlan) elementValue;
        } else {
            throw new ReaderException("Bad root element");
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void buildUnmarshallerSchema() throws Exception {
        SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        // Build the composite schema for validation.

        InputStream planSchemaStream = this.schemaStreamFactory.getSchemaStream(DEPLOYMENT_PLAN_XSD);
        InputStream m2SchemaStream = this.schemaStreamFactory.getSchemaStream(MAVEN2_DEPLOYMENT_PLAN_XSD);
        InputStream urlSchemaStream = this.schemaStreamFactory.getSchemaStream(URL_DEPLOYMENT_PLAN_XSD);
        InputStream obrSchemaStream = this.schemaStreamFactory.getSchemaStream(OBR_DEPLOYMENT_PLAN_XSD);

        SAXSource planSchemaSource = new SAXSource(new InputSource(planSchemaStream));
        SAXSource m2SchemaSource = new SAXSource(new InputSource(m2SchemaStream));
        SAXSource urlSchemaSource = new SAXSource(new InputSource(urlSchemaStream));
        SAXSource obrSchemaSource = new SAXSource(new InputSource(obrSchemaStream));
        Source[] sources = new Source[] {
                    planSchemaSource,
                    m2SchemaSource,
                    urlSchemaSource,
                    obrSchemaSource};
        this.unmarshallerSchema = sf.newSchema(sources);
    }

}
